const char *anatype = "ESD";

void AnalysisTrainGrid()
{
// Analysis using ESD/MC
// Automatically generated analysis steering macro executed in grid subjobs

// load base root libraries
   gSystem->Load("libTree");
   gSystem->Load("libGeom");
   gSystem->Load("libVMC");
   gSystem->Load("libPhysics");

// Load analysis framework libraries
   if (!SetupPar("STEERBase")) return;
   if (!SetupPar("ESD")) return;
   if (!SetupPar("AOD")) return;
   if (!SetupPar("ANALYSIS")) return;
   if (!SetupPar("ANALYSISalice")) return;
   if (!SetupPar("CORRFW")) return;

// Compile other par packages
   if (!SetupPar("PWG3base.par")) return;
   if (!SetupPar("PWG3muon.par")) return;
   if (!SetupPar("PWG3vertexingHF.par")) return;
// include path
   gSystem->AddIncludePath("-I$ALICE_ROOT/include");


// analysis source to be compiled at runtime (if any)

// connect to AliEn and make the chain
   if (!TGrid::Connect("alien://")) return;
   TChain *chain = CreateChain("wn.xml", anatype);

// read the analysis manager from file
   TFile *file = TFile::Open("analysis.root");
   if (!file) return;
   TIter nextkey(file->GetListOfKeys());
   AliAnalysisManager *mgr = 0;
   TKey *key;
   while ((key=(TKey*)nextkey())) {
      if (!strcmp(key->GetClassName(), "AliAnalysisManager"))
         mgr = (AliAnalysisManager*)file->Get(key->GetName());
   };
   if (!mgr) {
      ::Error("AnalysisTrainGrid", "No analysis manager found in file analysis.root");
      return;
   }

   mgr->PrintStatus();
   mgr->StartAnalysis("localfile", chain);
}

//________________________________________________________________________________
TChain* CreateChain(const char *xmlfile, const char *type="ESD")
{
// Create a chain using url's from xml file
   TString treename = type;
   treename.ToLower();
   treename += "Tree";
   printf("***************************************\n");
   printf("    Getting chain of trees %s\n", treename.Data());
   printf("***************************************\n");
   TAlienCollection *coll = TAlienCollection::Open(xmlfile);
   if (!coll) {
      ::Error("CreateChain", "Cannot create an AliEn collection from %s", xmlfile);
      return NULL;
   }
   TChain *chain = new TChain(treename);
   coll->Reset();
   while (coll->Next()) chain->Add(coll->GetTURL(""));
   if (!chain->GetNtrees()) {
      ::Error("CreateChain", "No tree found from collection %s", xmlfile);
      return NULL;
   }
   return chain;
}

//________________________________________________________________________________
Bool_t SetupPar(const char *package) {
// Compile the package and set it up.
   TString pkgdir = package;
   pkgdir.ReplaceAll(".par","");
   gSystem->Exec(Form("tar xvzf %s.par", pkgdir.Data()));
   TString cdir = gSystem->WorkingDirectory();
   gSystem->ChangeDirectory(pkgdir);
   // Check for BUILD.sh and execute
   if (!gSystem->AccessPathName("PROOF-INF/BUILD.sh")) {
      printf("*******************************\n");
      printf("*** Building PAR archive    ***\n");
      printf("*******************************\n");
      if (gSystem->Exec("PROOF-INF/BUILD.sh")) {
         ::Error("SetupPar", "Cannot build par archive %s", pkgdir.Data());
         gSystem->ChangeDirectory(cdir);
         return kFALSE;
      }
   } else {
      ::Error("SetupPar","Cannot access PROOF-INF/BUILD.sh for package %s", pkgdir.Data());
      gSystem->ChangeDirectory(cdir);
      return kFALSE;
   }
   // Check for SETUP.C and execute
   if (!gSystem->AccessPathName("PROOF-INF/SETUP.C")) {
      printf("*******************************\n");
      printf("***    Setup PAR archive    ***\n");
      printf("*******************************\n");
      gROOT->Macro("PROOF-INF/SETUP.C");
   } else {
      ::Error("SetupPar","Cannot access PROOF-INF/SETUP.C for package %s", pkgdir.Data());
      gSystem->ChangeDirectory(cdir);
      return kFALSE;
   }
   // Restore original workdir
   gSystem->ChangeDirectory(cdir);
   return kTRUE;
}
